@@ -6,7 +6,6 @@
 # Author D.L.
 # --------------------------------------------------------------------------
 # require(PowerTOST)
-# source("./R/sampsiz.R")
 # source("./R/sampsiz2.R")
 # source("./R/sampsiz_n0.R")
 # source("./R/power.R")
@@ -15,8 +14,8 @@ power.2stage <- function(method=c("B","C"), alpha0=0.05, alpha=c(0.0294,0.0294),
                          n1, GMR, CV, targetpower=0.8, 
                          pmethod=c("nct","exact", "shifted"),
                          usePE=FALSE, Nmax=Inf, min.n2=0, theta0, theta1, theta2,  
-                         npct=c(0.05, 0.5, 0.95), nsims=1e5, setseed=TRUE, 
-                         print=TRUE, details=TRUE)
+                         npct=c(0.05, 0.5, 0.95), nsims, setseed=TRUE, 
+                         print=TRUE, details=FALSE)
 {
   if (missing(CV)) stop("CV must be given.")
   if (CV<=0)       stop("CV must be >0.")
@@ -36,19 +35,23 @@ power.2stage <- function(method=c("B","C"), alpha0=0.05, alpha=c(0.0294,0.0294),
   
   if (n1>Nmax) stop("n1>Nmax doestn't make sense!")
   
+  if(missing(nsims)){
+    nsims <- 1E5
+    if(theta0<=theta1 | theta0>=theta2) nsims <- 1E6
+  }
+  
   if(min.n2!=0 & min.n2<2) stop("min.n2 has to be at least +2.")
   # make even (round up)
   if( min.n2%%2 != 0) {
     min.n2 <- min.n2 + min.n2%%2
     message("min.n2 rounded to even", min.n2)
   }
-  
   # check if Potvin B or C
   method  <- match.arg(method)
   # check if power calculation method is nct or exact
   pmethod <- match.arg(pmethod)
   
-  if(print & details){
+  if(details){
     cat(nsims,"sims. Stage 1")
   }
   # start timer
@@ -121,7 +124,7 @@ power.2stage <- function(method=c("B","C"), alpha0=0.05, alpha=c(0.0294,0.0294),
   rm(BE1, hw, lower, upper)
   
   # time for stage 1
-  if(print & details){
+  if(details){
     cat(" - Time consumed (secs):\n")
     print(round((proc.time()-ptm),1))
   }
@@ -134,7 +137,7 @@ power.2stage <- function(method=c("B","C"), alpha0=0.05, alpha=c(0.0294,0.0294),
   
   # Maybe we are already done with stage 1
   if(length(pes_tmp)>0){
-    if(print & details){
+    if(details){
       cat("Keep calm. Sample sizes for stage 2 (", length(pes_tmp),
           " studies)\n", sep="")
       cat("will be estimated. May need some time.\n")
@@ -175,7 +178,7 @@ power.2stage <- function(method=c("B","C"), alpha0=0.05, alpha=c(0.0294,0.0294),
     # some upper bound due to numerics?
     #n2  <- ifelse(n2>100000, 100000, n2) # may not necessary
     
-    if(print & details){
+    if(details){
       cat("Time consumed (secs):\n")
       print(round((proc.time()-ptms),1))
     }
@@ -231,10 +234,11 @@ power.2stage <- function(method=c("B","C"), alpha0=0.05, alpha=c(0.0294,0.0294),
   # take care of memory
   rm(pes_tmp, mses_tmp)
   # the return list
-  res <- list(method=method, alpha0=ifelse(method=="C",alpha0,NA), alpha=alpha, 
+  res <- list(design="2x2 crossover",
+              method=method, alpha0=ifelse(method=="C",alpha0,NA), alpha=alpha, 
               CV=CV, n1=n1, GMR=GMR, targetpower=targetpower, pmethod=pmethod, 
               theta0=exp(mlog), theta1=theta1, theta2=theta2, usePE=usePE, 
-              Nmax=Nmax, nsims=nsims,
+              Nmax=Nmax, min.n2=min.n2, nsims=nsims,
               # results 
               pBE=sum(BE)/nsims,
               # Dec 2014 changed to
@@ -248,38 +252,23 @@ power.2stage <- function(method=c("B","C"), alpha0=0.05, alpha=c(0.0294,0.0294),
               # now it is 
               pct_s2=100*sum(ntot>n1)/nsims, 
               # which simply means all those with n2>0
-              nmean=mean(ntot), nrange=range(ntot), nperc=quantile(ntot, p=npct))
-  # output
-  if (print) {
-    if (details){
-      cat("Total time consumed (secs):\n")
-      print(round((proc.time()-ptm),1))
-      cat("\n")
-    }
-    cat("Method ", method,":", sep="")
-    if (method=="C") cat(" alpha0 = ", alpha0, ",",sep="")
-    cat(" alpha (s1/s2) =", alpha[1], alpha[2], "\n")
-    cat("Target power in power monitoring and sample size est. = ", 
-        targetpower,"\n",sep="")
-    cat("BE margins = ", theta1," ... ", theta2,"\n", sep="")
-    cat("CV = ",CV,"; n(stage 1)= ",n1,"; GMR = ",GMR, "\n", sep="")
-    if(usePE) cat("PE and mse of stage 1 in sample size est. used\n") else {
-      cat("GMR =", GMR, "and mse of stage 1 in sample size est. used\n")}
-    cat("Futility criterion Nmax = ",Nmax,"\n", sep="")
-    cat("\n",nsims," sims at theta0 = ", theta0, sep="")
-    if(theta0<=theta1 | theta0>=theta2) cat(" (p(BE)='alpha').\n") else { 
-       cat(" (p(BE)='power').\n")}
-    cat("p(BE)    = ", res$pBE,"\n", sep="")
-    cat("p(BE) s1 = ", res$pBE_s1,"\n", sep="")
-    cat("Studies in stage 2 = ", round(res$pct_s2,2), "%\n", sep="")
-    cat("\nDistribution of n(total)\n")
-    cat("- mean (range) = ", round(res$nmean,1)," (", res$nrange[1]," ... ",
-        res$nrange[2],")\n", sep="")
-    cat("- percentiles\n")
-    print(res$nperc)
-    cat("\n")
-  } 
+              nmean=mean(ntot), nrange=range(ntot), nperc=quantile(ntot, p=npct)
+              #, ntot=ntot # experimental: return also all sample sizes
+              )
+
+  # table object summarizing the discrete distri of ntot
+  # only if usePE=FALSE or if usePE=TRUE then Nmax must be finite?
+  if (usePE==FALSE | (usePE==TRUE & is.finite(Nmax))){
+    res$ntable <- table(ntot)
+  }
   
-  if (print) return(invisible(res)) else return(res)
+  if (details){
+    cat("Total time consumed (secs):\n")
+    print(round((proc.time()-ptm),1))
+    cat("\n")
+  }
+  
+  class(res) <- c("pwrtsd", "list")
+  return(res)
   
 } #end function
