@@ -1,62 +1,67 @@
-# sample size search start values
 # ----- helper functions ---------------------------------------------------
+# sample size search start values
+#
 # Sample size for a desired power, large sample approx.
-# 
-# original 'large sample' sample size formula
+# original 'large sample' sample size formula if diffmthreshold=0
 # author D. Labes
+# vectorizes properly if diffm is scalar and se a vector or both are vectors
 .sampleN00 <- function(alpha=0.05, targetpower=0.8, ltheta1, ltheta2, diffm, 
                        se, steps=2, bk=2, diffmthreshold=0.04)
 {
-  
   z1 <- qnorm(1-alpha)
+  # within diffmthreshold diffm will be regarded as 0
   # value diffmthreshold=0.04 corresponds roughly to log(0.96)
-  # with lower values there are many steps around between 0.95 and 1
-  # in sampleN.TOST, but no longer used in sampleN.TOST
-  if (abs(diffm)>diffmthreshold) z2 <- qnorm(targetpower) else {
-    z2 <- qnorm(1-(1-targetpower)/2) # #1-beta/2 for diffm ~0 (log(theta0=1))
-    diffm <- 0
-  }
+  # with lower values there are many steps in sample size around between 0.95 and 1.
+  # no longer used. Zhangs formula instead
+  # vectorized solution
+  z2    <- ifelse(abs(diffm)>diffmthreshold, qnorm(targetpower), 
+                  qnorm(1-(1-targetpower)/2))
+  diffm <- ifelse(abs(diffm)>diffmthreshold, diffm, 0)
+
   dn <- ifelse(diffm<0, diffm-ltheta1, diffm-ltheta2)
+  # return Inf if diffm outside, dn=0 accomplishes that
+  dn <- ifelse((diffm-ltheta1)<1.25e-5 | (ltheta2-diffm)<1.25e-5, 0, dn)
   n0 <- (bk/2)*((z1+z2)*(se*sqrt(2)/dn))^2
   
   # round up to next even >=
   # seems Golkowski has used simple round
   n0 <- steps*ceiling(n0/steps)
-  # return Inf if diffm outside
-  n0 <- ifelse ((diffm-ltheta1)<1.25e-5 | (ltheta2-diffm)<1.25e-5, Inf, n0)
-  
+
   return(n0)
   
 }
 
+# ----------------------------------------------------------------------------
 # 'large sample' sample size with one additional step via t-distribution
 # author D. Labes
 # bk = design constant, see known.designs()
 .sampleN0 <- function(alpha=0.05, targetpower=0.8, ltheta1, ltheta2, diffm, 
                       se, steps=2, bk=2, diffmthreshold=0.04)
 {
+  # paranoia? swap
+  if (ltheta2<ltheta1) {h <- ltheta2; ltheta2 <- ltheta1; ltheta1 <-h}
   
   z1 <- qnorm(1-alpha)
   # value diffmthreshold=0.04 corresponds roughly to log(0.96)
-  # with lower values there are many steps around between 0.95 and 1
-  # in sampleN.TOST
-  if (abs(diffm)>diffmthreshold) z2 <- qnorm(targetpower) else {
-    z2 <- qnorm(1-(1-targetpower)/2) # #1-beta/2 for diffm ~0 (log(theta0=1))
-    diffm <- 0
-  }
-
+  # with lower values or 0 there are many steps around between 0.95 and 1
+  # in sampleN.TOST, no longer used but Zhang's formula instead
+  # vectorized solution
+  z2    <- ifelse(abs(diffm)>diffmthreshold, qnorm(targetpower), qnorm(1-(1-targetpower)/2))
+  diffm <- ifelse(abs(diffm)>diffmthreshold, diffm, 0)
+  
   dn <- ifelse(diffm<0, diffm-ltheta1, diffm-ltheta2)
+  # return Inf if diffm outside, dn=0 accomplishes that
+  dn <- ifelse((diffm-ltheta1)<1.25e-5 | (ltheta2-diffm)<1.25e-5, 0, dn)
   n0 <- (bk/2)*((z1+z2)*(se*sqrt(2)/dn))^2
   
-  if (n0>2){
+  if (is.finite(n0) & n0>2){
     # make another step with t-distri
     n0 <- ceiling(n0)
     z1 <- qt(1-alpha, df=n0-2)
-    if (abs(diffm)>diffmthreshold) z2 <- qt(targetpower, df=n0-2) else {
-      z2 <- qt(1-(1-targetpower)/2, df=n0-2)  
-      diffm <- 0
-    }
-    
+    z2    <- ifelse(abs(diffm)>diffmthreshold, qt(targetpower, df=n0-2), 
+                    qt(1-(1-targetpower)/2, df=n0-2))
+    diffm <- ifelse(abs(diffm)>diffmthreshold, diffm, 0)
+
     dn <- ifelse(diffm<0, diffm-ltheta1, diffm-ltheta2)
     n0 <- (bk/2)*((z1+z2)*(se*sqrt(2)/dn))^2
     
@@ -68,7 +73,7 @@
   return(n0)
 }
 
-
+# ----------------------------------------------------------------------------
 # Paul Zhang (2003)
 # A Simple Formula for Sample Size Calculation in Equivalence Studies 
 # Journal of Biopharmaceutical Statistics, 13:3, 529-538
@@ -84,19 +89,20 @@
   z2 <- abs(qnorm((1-fz)*beta))
   
   dn <- ifelse(diffm<0, diffm-ltheta1, diffm-ltheta2)
+  # return Inf if diffm outside, dn=0 accomplishes that
+  dn <- ifelse((diffm-ltheta1)<1.25e-5 | (ltheta2-diffm)<1.25e-5, 0, dn)
   n0 <- (bk/2)*((z1+z2)*(se*sqrt(2)/dn))^2
   
   # make an even multiple of step (=2 in case of 2x2 cross-over)
   n0 <- steps*trunc(n0/steps)
-  # return Inf if diffm on or outside BE acceptance range
-  n0 <- ifelse ((diffm-ltheta1)<1.25e-5 | (ltheta2-diffm)<1.25e-5, Inf, n0)
-  
+
   return(n0)
 }
 
 # -----------------------------------------------------------------------------
-# variant of 'large sample' formula with 'smooth' transition from beta/2 to beta
-# and a threshold
+# variant of Zhangs 'large sample' formula with 'smooth' transition from beta/2 
+# to beta and a threshold
+# vectorizes properly if diffm is scalar and se a vector or both are vectors
 .sampleN0_3 <- function(alpha=0.05, targetpower=0.8, ltheta1, ltheta2, diffm, 
                         se, steps=2, bk=2)
 {
@@ -117,12 +123,13 @@
   z2 <- qnorm(p2)
   # difference for denominator
   dn <- ifelse(diffm<0, diffm-ltheta1, diffm-ltheta2)
+  # return Inf if diffm outside, dn=0 accomplishes that
+  dn <- ifelse((diffm-ltheta1)<1.25e-5 | (ltheta2-diffm)<1.25e-5, 0, dn)
+  
   n0 <- (bk/2)*((z1+z2)*(se*sqrt(2)/dn))^2
   # make an even multiple of steps (=2 in case of 2x2 cross-over)
   n0 <- steps*trunc(n0/steps)
-  # return Inf if diffm on or outside BE acceptance range
-  n0 <- ifelse ((diffm-ltheta1)<1.25e-5 | (ltheta2-diffm)<1.25e-5, Inf, n0)
-  
+
   return(n0)
   
 }
